# -*- encoding: utf-8 -*-

import paramiko
import base64
import stat

from vpnchooser.applicaton import app
from vpnchooser.objects import Rule


class Client(object):
    """
    Client to connect to the router and
    synchronise with the data.
    """

    def __init__(self):
        self.rule_location = '/tmp/vpnchooser_rules/rules.txt'

    def connect(self):
        """
        Connects the client to the server and
        returns it.
        """
        key = paramiko.RSAKey(data=base64.b64decode(
            app.config['SSH_HOST_KEY']
        ))
        client = paramiko.SSHClient()
        client.get_host_keys().add(
            app.config['SSH_HOST'],
            'ssh-rsa',
            key
        )
        client.connect(
            app.config['SSH_HOST'],
            username=app.config['SSH_USER'],
            password=app.config['SSH_PASSWORD'],
        )
        return client

    def _create_directory_structure(self):
        sftp = self.client.open_sftp()
        rules = [
            rule_string
            for rule_string in
            self.rule_location.split("/")
            if len(rule_string) > 0
        ]
        checked_rules = []
        for rule in rules[:-1]:
            check_path = '/' + '/'.join(checked_rules)
            checked_rules.append(rule)
            if rule not in sftp.listdir(
                check_path
            ):
                sftp.mkdir('/' + '/'.join(checked_rules))
        sftp.close()

    @property
    def server_rules(self):
        """
        Reads the server rules from the client
        and returns it.
        """
        sftp = self.client.open_sftp()
        try:
            rule_path = self.rule_location
            try:
                stat_entry = sftp.stat(rule_path)
                if stat.S_ISDIR(stat_entry.st_mode):
                    sftp.rmdir(rule_path)
                    return []
            except IOError:
                return []
            with sftp.open(rule_path, 'r') as file_handle:
                data = file_handle.read()
            return self._parse(data)
        finally:
            sftp.close()

    @staticmethod
    def _parse(data: str) -> list:
        """
        Parses the given data string and returns
        a list of rule objects.
        """
        if isinstance(data, bytes):
            data = data.decode('utf-8')
        lines = (
            item for item in
            (item.strip() for item in data.split('\n'))
            if len(item) and not item.startswith('#')
        )
        rules = []
        for line in lines:
            rules.append(
                Rule.parse(line)
            )
        return rules

    def sync(self, rules: list):
        """
        Synchronizes the given ruleset with the
        one on the server and adds the not yet
        existing rules to the server.
        :type rules: collections.Iterable[Rule]
        """
        self.client = self.connect()
        try:
            server_rules = set(self.server_rules)
            rules = set(rules)
            to_remove_rules = server_rules.difference(rules)
            to_add_rules = rules.difference(server_rules)
            for to_remove_rule in to_remove_rules:
                stdin, stdout, stderr = self.client.exec_command(
                    to_remove_rule.remove_command
                )
                stdout.read()
                stderr.read()
            for to_add_rule in to_add_rules:
                stdin, stdout, stderr = self.client.exec_command(
                    to_add_rule.add_command
                )
                stdout.read()
                stderr.read()

            if len(to_remove_rules) or len(to_add_rules):
                self._write_to_server(rules)
                stdin, stdout, stderr = self.client.exec_command(
                    'ip route flush cache'
                )
                stdout.read()
                stderr.read()
        finally:
            self.client.close()

    def _write_to_server(self, rules: list):
        """
        Writes the given ruleset to the
        server configuration file.
        :type rules: collections.Iterable[Rule]
        """
        self._create_directory_structure()
        config_data = '\n'.join(rule.config_string for rule in rules)
        sftp = self.client.open_sftp()
        try:
            with sftp.open(self.rule_location, 'w') as file_handle:
                file_handle.write(config_data)
                file_handle.write('\n')
        finally:
            sftp.close()



# -*- encoding: utf-8 -*-

from flask.ext.restful import (
    Resource,
    fields,
    marshal_with,
    abort,
    url_for
)
from flask.ext.restful.reqparse import RequestParser

from vpnchooser.helpers import (
    require_admin, require_login, current_user
)
from vpnchooser.helpers.fields import AbsoluteUrl
from vpnchooser.db import session, User


parser = RequestParser()
parser.add_argument(
    'name', type=str,
    required=False,
)
parser.add_argument(
    'password', type=str,
    required=False,
)
parser.add_argument(
    'is_admin', type=bool,
    required=False,
)


resource_fields = {
    'name': fields.String,
    'is_admin': fields.Boolean,
    'api_key': fields.String,
    'self': AbsoluteUrl('user', data_func=lambda obj: {
        'user_name': obj.name
    }),
}


class AbstractUserResource(Resource):
    """
    Abstract of the resource.
    """

    @staticmethod
    def _get_by_username(user_name: str) -> User:
        return session.query(User).filter(
            User.name == user_name
        ).first()

    @staticmethod
    def update(user: User) -> User:
        args = parser.parse_args()
        if not user.name and args.name:
            user.name = args.name
        if current_user().is_admin and args.is_admin is not None:
            user.is_admin = args.is_admin
        if args.password:
            user.password = args.password
            user.generate_api_key()
        return user


class UserResource(AbstractUserResource):
    """
    The resource to access a user resource.
    """

    def _get_or_abort(self, user_name: str):
        user = self._get_by_username(user_name)
        if user is None:
            abort(404)
        else:
            pass
        return user

    @require_login
    @marshal_with(resource_fields)
    def get(self, user_name: str) -> User:
        """
        Gets the User Resource.
        """
        user = current_user()
        if user.is_admin or user.name == user_name:
            return self._get_or_abort(user_name)
        else:
            abort(403)

    @require_login
    @marshal_with(resource_fields)
    def put(self, user_name: str) -> User:
        """
        Updates the User Resource with the
        name.
        """
        current = current_user()
        if current.name == user_name or current.is_admin:
            user = self._get_or_abort(user_name)
            self.update(user)
            session.commit()
            session.add(user)
            return user
        else:
            abort(403)

    @require_admin
    def delete(self, user_name: str):
        """
        Deletes the resource with the given name.
        """
        user = self._get_or_abort(user_name)
        session.delete(user)
        session.commit()
        return '', 204


class UserListResource(AbstractUserResource):
    """
    List resource for the user.
    """

    @require_login
    @marshal_with(resource_fields)
    def get(self):
        user = current_user()
        query = session.query(User)
        if not user.is_admin:
            query = query.filter(User.name == user.name)
        return list(query)

    @require_admin
    @marshal_with(resource_fields)
    def post(self) -> User:
        args = parser.parse_args()
        user = self._get_by_username(args.name)
        if user is not None:
            abort(409, message='User already exists')
        user = User()
        self.update(user)
        session.add(user)
        session.commit()
        return user, 201, {
            'Location': url_for('user', user_name=user.name)
        }



# -*- encoding: utf-8 -*-

from celery import Celery


def make_celery(app):
    celery = Celery(
        app.import_name,
    )
    TaskBase = celery.Task

    class ContextTask(TaskBase):
        abstract = True

        def __call__(self, *args, **kwargs):
            with app.app_context():
                return TaskBase.__call__(self, *args, **kwargs)

    celery.Task = ContextTask
    return celery



# -*- encoding: utf-8 -*-

import os
import hashlib

from passlib.hash import pbkdf2_sha512
from sqlalchemy.ext.hybrid import hybrid_property

from .database import db


class User(db.Model):
    """
    A user resource to be able to authenticate.
    """

    __tablename__ = 'user'

    name = db.Column(db.Unicode(255), primary_key=True)

    _password = db.Column('password', db.Unicode(512), nullable=False)

    @hybrid_property
    def password(self):
        return self._password

    @password.setter
    def password(self, password: str):
        self._password = pbkdf2_sha512.encrypt(password)

    def check(self, password: str) -> bool:
        """
        Checks the given password with the one stored
        in the database
        """
        return (
            pbkdf2_sha512.verify(password, self.password) or
            pbkdf2_sha512.verify(password,
                                 pbkdf2_sha512.encrypt(self.api_key))
        )

    is_admin = db.Column(db.Boolean, default=False,
                         server_default='false')

    _api_key = db.Column('api_key', db.Unicode(512), nullable=False)

    def generate_api_key(self):
        self._api_key = hashlib.new('sha512',
                                    os.urandom(512)).hexdigest()

    @hybrid_property
    def api_key(self):
        if self._api_key is None:
            self.generate_api_key()
        return self._api_key

    @api_key.setter
    def api_key(self, api_key):
        self._api_key = api_key

    @api_key.expression
    def api_key(cls):
        return cls._api_key



# -*- encoding: utf-8 -*-

from functools import wraps

from flask import request, Response, g

from vpnchooser.db import session, User
from .auth import current_user


def authenticate():
    """Sends a 401 response that enables basic auth"""
    return Response(
        'Could not verify your access level for that URL.\n'
        'You have to login with proper credentials', 401,
        # {'WWW-Authenticate': 'Basic realm="Login Required"'}
        {}
    )


def require_login(func):
    """
    Function wrapper to signalize that a login is required.
    """

    @wraps(func)
    def decorated(*args, **kwargs):
        auth = request.authorization
        if not auth:
            return authenticate()

        user = session.query(User).filter(
            User.name == auth.username
        ).first()
        if user and user.check(auth.password):
            g.user = user
            return func(*args, **kwargs)
        else:
            return authenticate()

    return decorated


def require_admin(func):
    """
    Requires an admin user to access this resource.
    """

    @wraps(func)
    @require_login
    def decorated(*args, **kwargs):
        user = current_user()
        if user and user.is_admin:
            return func(*args, **kwargs)
        else:
            return Response(
                'Forbidden', 403
            )

    return decorated



# -*- encoding: utf-8 -*-

from vpnchooser.connection import Client
from vpnchooser.db import session, Device

from .applicaton import celery


class Syncer:
    """
    Handler to sync the current dataset
    to the rule set of the client.
    """

    def __init__(self):
        self.client = Client()

    def sync(self):
        return self.client.sync(self.rules)

    @property
    def rules(self):
        qry = session.query(Device).filter(Device.vpn_id != None)
        return [
            item.rule
            for item in qry
        ]


@celery.task(name='vpnchooser.sync')
def sync():
    """
    Synchronizes the current dataset from the database
    with the ip rule set on the router system.
    """
    syncer = Syncer()
    return syncer.sync()



# -*- encoding: utf-8 -*-

from urllib.parse import urlparse, urlunparse

from flask.ext.restful.fields import Url
from flask.helpers import url_for


class AbsoluteUrl(Url):
    def __init__(self,
                 endpoint,
                 absolute=True,
                 scheme=None,
                 data_func=None):
        super(AbsoluteUrl, self).__init__(
            endpoint=endpoint,
            absolute=absolute,
            scheme=scheme,
        )
        self.data_func = data_func

    def output(self, key, obj):
        if self.data_func:
            data = self.data_func(obj)
            o = urlparse(
                url_for(self.endpoint, _external=self.absolute, **data)
            )
            if self.absolute:
                scheme = self.scheme if self.scheme is not None else o.scheme
                return urlunparse(
                    (scheme, o.netloc, o.path, "", "", ""))
            return urlunparse(("", "", o.path, "", "", ""))
        else:
            return super(AbsoluteUrl, self).output(key, obj)


class NullableAbsoluteUrl(AbsoluteUrl):
    def output(self, key, obj):
        try:
            return super(NullableAbsoluteUrl, self).output(key, obj)
        except Exception:
            return None



# -*- encoding: utf-8 -*-

from getpass import getpass

from pkg_resources import resource_string
from paramiko.client import (
    SSHClient,
    MissingHostKeyPolicy,
)


class ConfigurationGenerator(MissingHostKeyPolicy):
    """
    Generates a config file from given input.
    """

    def __init__(self, file_location):
        self.file_location = file_location

    def request_host(self):
        self.host = input("Router SSH host: ")

    def request_username(self):
        self.username = input("Router SSH username: ") or "root"

    def request_password(self):
        self.password = getpass("Router SSH password: ")

    def request_database(self):
        print("(see http://docs.sqlalchemy.org/en/latest/core/engines.html )")
        self.database = input("Database url (sqlite:///test.db): ")

    def request_redis(self):
        redis = input("Host and Port for redis (127.0.0.1:6379): ")
        self.redis = redis or "127.0.0.1:6379"

    def request_data(self):
        self.request_host()
        self.request_username()
        self.request_password()
        self.request_database()
        self.request_redis()

    def generate(self):
        print("Generating Configuration file")
        self.request_data()
        print("Connecting host to get ssh host key and check your "
              "credentials.")
        self.verify()
        self.write_config()

    def write_config(self):
        config_bytes = resource_string('vpnchooser', 'template.cfg')
        config = config_bytes.decode('utf-8')
        written_config = config % dict(
            username=self.username,
            password=self.password,
            host=self.host,
            host_key=self.host_key,
            broker_url=self.redis,
            database=self.database
        )
        with open(self.file_location, 'w') as f:
            f.write(written_config)

    def verify(self):
        client = SSHClient()
        client.set_missing_host_key_policy(self)
        client.connect(
            self.host,
            username=self.username,
            password=self.password,
        )

    def missing_host_key(self, client, hostname, key):
        """
        Called when an `.SSHClient` receives a server key for a server that
        isn't in either the system or local `.HostKeys` object.  To accept
        the key, simply return.  To reject, raised an exception (which will
        be passed to the calling application).
        """
        self.host_key = key.get_base64()
        print("Fetched key is: %s" % self.host_key)
        return


class DockerConfigurationGenerator(ConfigurationGenerator):
    """
    Configuration generator for generating the configuration
    file for docker containers.
    """

    def request_database(self):
        self.database = "sqlite:////data/vpnchooser.db"

    def request_redis(self):
        self.redis = "redis:6379"



# -*- encoding: utf-8 -*-

from vpnchooser.objects import Rule

from .database import db


class Device(db.Model):
    """
    A device in the network.
    """

    __tablename__ = 'device'

    id = db.Column(db.Integer, primary_key=True)

    ip = db.Column(db.Unicode(255), nullable=False)

    name = db.Column(db.Unicode(255), nullable=False)

    type = db.Column(db.Unicode(255), nullable=True)

    vpn_id = db.Column(
        db.Integer,
        db.ForeignKey('vpn.id'),
        nullable=True
    )

    vpn = db.relationship(
        'Vpn',
        uselist=False,
        lazy=False,
        backref=db.backref('devices',
                           uselist=True,
                           lazy=True)
    )

    @property
    def rule(self) -> Rule:
        if self.vpn is None:
            return None

        rule = Rule()
        rule.ip = '{ip}/32'.format(
            ip=self.ip
        )
        rule.table = self.vpn.table
        return rule



# -*- encoding: utf-8 -*-

from paramiko import SSHClient, SFTPClient

from .objects import Rule


class RulesQuery(object):

    RULE_PATH = None

    def __init__(self, connection: SSHClient):
        self._ssh = connection
        self._sftp = None
        self._reset()

    def _load_rules(self):
        """
        Loads the rules from the SSH-Connection
        """
        with self._sftp_connection.open(self.RULE_PATH) as file:
            data = file.read()
        lines = (
            line.strip()
            for line in data.split('\n')
        )
        rule_strings = (
            line for line in lines
            if len(line) > 0
        )
        rules = (
            Rule.parse(rule_string)
            for rule_string in rule_strings
        )
        self._rules = [
            rule
            for rule in rules
            if rule is not None
        ]

    def _reset(self):
        """
        Resets the internal state to download
        the data from the router of the currently
        valid rules.
        """
        self._rules = None

    @property
    def rules(self):
        if self._rules is None:
            self._load_rules()
        return self._rules

    @property
    def _sftp_connection(self) -> SFTPClient:
        if self._sftp is None:
            self._sftp = self._ssh.open_sftp()
        return self._sftp

    def __del__(self):
        try:
            self._sftp.close()
        except Exception:
            pass
        try:
            self._ssh.close()
        except Exception:
            pass

    def _exec_command(self, command: str):
        """
        Executes the command and closes the handles
        afterwards.
        """
        stdin, stdout, stderr = self._ssh.exec_command(command)
        # Clearing the buffers
        stdout.read()
        stderr.read()
        stdin.close()

    def sync(self, rules: list):
        """
        Synchronizes the given rules with the server
        and ensures that there are no old rules active
        which are not in the given list.
        """
        self._reset()
        old_rules = self.rules
        to_delete_rules = [
            rule for rule in old_rules
            if rule not in rules
        ]
        new_rules = [
            rule for rule in rules
            if rule not in old_rules
        ]

        for new_rule in new_rules:
            assert isinstance(new_rule, Rule)
            self._exec_command(new_rule.add_command)
        for to_delete_rule in to_delete_rules:
            assert isinstance(to_delete_rule, Rule)
            self._exec_command(
                to_delete_rule.remove_command
            )
        self._update(rules)

    def _update(self, rules: list):
        """
        Updates the given rules and stores
        them on the router.
        """
        self._rules = rules
        to_store = '\n'.join(
            rule.config_string
            for rule in rules
        )
        sftp_connection = self._sftp_connection
        with sftp_connection.open(self.RULE_PATH, mode='w') as file_handle:
            file_handle.write(to_store)



# -*- encoding: utf-8 -*-

from urllib.parse import urlparse
from werkzeug.exceptions import NotFound

from vpnchooser.applicaton import app


def id_from_url(url, param_name: str) -> int:
    """
    Parses an object and tries to extract a url.
    Tries to parse if a resource_url has been given
    it as a url.
    :raise ValueError: If no id could be extracted.
    """
    if url is None:
        raise ValueError('url is none')
    elif isinstance(url, int):
        # Seems to already be the url.
        return url
    if not url:
        raise ValueError('Seems to be empty')

    try:
        return int(url)
    except ValueError:
        pass

    parsed = urlparse(url)
    try:
        resource_url = app.url_map.bind(parsed.netloc).match(
            parsed.path
        )
    except NotFound:
        raise ValueError('No URL found')

    if param_name in resource_url[1]:
        return resource_url[1][param_name]
    else:
        raise ValueError(
            'Parameter {name} could not be extracted'.format(
                name=param_name
            )
        )



# -*- encoding: utf-8 -*-

from .database import db


class Vpn(db.Model):
    """
    A vpn which is configured to a specific
    table rule.
    """

    __tablename__ = 'vpn'

    id = db.Column(db.Integer, primary_key=True)

    name = db.Column(db.Unicode(255), nullable=False)

    description = db.Column(db.Unicode(255), nullable=False)

    table = db.Column(db.Unicode(255), nullable=False)



# -*- encoding: utf-8 -*-

from vpnchooser.applicaton import api

from .device import DeviceResource, DeviceListResource
from .vpn import VpnResource, VpnListResource
from .user import UserResource, UserListResource


api.add_resource(
    DeviceListResource,
    '/devices', '/devices/',
    endpoint='device_list'
)
api.add_resource(
    DeviceResource,
    '/devices/<int:device_id>',
    endpoint='device'
)
api.add_resource(
    VpnListResource,
    '/vpns', '/vpns/',
    endpoint='vpn_list')
api.add_resource(
    VpnResource,
    '/vpns/<int:vpn_id>',
    endpoint='vpn'
)
api.add_resource(
    UserListResource,
    '/users', '/users/',
    endpoint='user_list'
)
api.add_resource(
    UserResource,
    '/users/<string:user_name>',
    endpoint='user'
)



# -*- encoding: utf-8 -*-
"""
Manager for the application.
"""

import sys
from getpass import getpass

from flask.ext.script import Manager
from celery.bin.worker import worker

from vpnchooser.applicaton import app, celery
from vpnchooser.syncer import sync as do_sync
from vpnchooser.db import db, session, User
from vpnchooser.cli import (
    ConfigurationGenerator,
    DockerConfigurationGenerator,
)


manager = Manager(app)


def _init_app(config=None):
    if config is None:
        app.config.from_envvar('FLASK_CONFIG_FILE')
    else:
        app.config.from_pyfile(config)
    celery.conf.update(app.config)
    celery.conf['BROKER_URL'] = app.config['CELERY_BROKER_URL']


@manager.command
@manager.option('-c', '--config', dest='config', default=None)
@manager.option('-h', '--host', dest='host', default=None)
@manager.option('-p', '--port', dest='port', default=None)
def runserver(config=None, host=None, port=None):
    _init_app(config)
    app.run(host=host, port=int(port))


@manager.command
@manager.option('-c', '--config', dest='config', default=None)
def init_db(config=None):
    _init_app(config)
    db.create_all()


@manager.command
@manager.option('-c', '--config', dest='config', default=None)
def sync(config=None):
    _init_app(config)
    do_sync()
    print("Synchronization complete")


@manager.command
@manager.option('-c', '--config', dest='config', default=None)
@manager.option('-u', '--username', dest='username')
@manager.option('-p', '--password', dest='password')
def create_admin(username, password, config=None):
    _init_app(config)
    user = User()
    user.name = username
    user.password = password
    user.is_admin = True
    user.generate_api_key()
    session.add(user)
    session.commit()


@manager.command
@manager.option('-c', '--config', dest='config', default=None)
@manager.option('-u', '--username', dest='username')
@manager.option('-p', '--password', dest='password', default=None)
def reset_password(username, password=None, config=None):
    _init_app(config)
    user = session.query(User).filter(User.name == username).first()
    if user is None:
        print('User %s not found' % username)
    if password is None:
        password = getpass('Enter new password: ')

    user.password = password
    session.commit()
    print('Password reset done for user %s' % username)


@manager.command
@manager.option('-c', '--config', dest='config', default=None)
def runcelery(config=None):
    _init_app(config)
    # Fix for setuptools generated scripts, so that it will
    # work with multiprocessing fork emulation.
    # (see multiprocessing.forking.get_preparation_data())
    if __name__ != '__main__':  # pragma: no cover
        sys.modules['__main__'] = sys.modules[__name__]
    from billiard import freeze_support

    freeze_support()
    worker(app=celery).run_from_argv('vpnchooser', argv=[
        '-B'
    ])


@manager.command
@manager.option(
    '-c',
    '--config',
    help='file location the config should be written to.'
)
@manager.option(
    '--docker',
    default=False,
    dest='docker',
    action='store_true',
    help='Wether the configuration should be populated by the default '
         'docker data.'
)
def generate_config(config, docker=False):
    if docker:
        generator = DockerConfigurationGenerator(config)
    else:
        generator = ConfigurationGenerator(config)
    generator.generate()


def main():
    manager.run()


if __name__ == '__main__':
    main()



# -*- encoding: utf-8 -*-

import re


parse_rule = re.compile('^(?P<ip>[0-9\.a-fA-F/]+) (?P<table>[0-9]+)$')


class Rule(object):
    """
    A rule which is bound to a table and an ip
    address range.
    """

    def __init__(self, ip=None, table=None):
        self.ip = ip
        self.table = table

    @property
    def add_command(self) -> str:
        if self.ip is None or self.table is None:
            return
        return "ip rule add from {ip} table {table}".format(
            **self.__dict__
        )

    @property
    def remove_command(self) -> str:
        if self.ip is None or self.table is None:
            return
        return "ip rule del from {ip} table {table}".format(
            **self.__dict__
        )

    @property
    def config_string(self) -> str:
        if self.ip is None or self.table is None:
            return
        return "{ip} {table}".format(
            **self.__dict__
        )

    @classmethod
    def parse(cls, line: str):
        match = parse_rule.match(line)
        if match is None:
            return None

        rule = cls()
        rule.ip = match.group('ip')
        rule.table = match.group('table')
        return rule

    def __eq__(self, other):
        if not isinstance(other, Rule):
            return False
        return other.ip == self.ip and other.table == self.table

    def __hash__(self):
        return self.ip.__hash__() + self.table.__hash__() ** 32



# -*- encoding: utf-8 -*-

from setuptools import setup, find_packages

requires = [
    'Flask==0.10.1',
    'Flask-RESTful==0.3.2',
    'Flask-Script==2.0.5',
    'Flask-SQLAlchemy==2.0',
    'paramiko',
    'passlib',
    'celery',
    'redis',
]

VERSION = '0.6.3'

setup(
    name='vpnchooser',
    version=VERSION,
    description='Web UI to switch clients between different ip rule tables',
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.4",
        "Framework :: Flask",
        "Topic :: System :: Networking",
    ],
    author='Christoph Brand',
    author_email='christoph@brand.rest',
    keywords=['vpn', 'network', 'ip'],
    packages=find_packages('src'),  # include all packages under src
    package_dir={'': 'src'},  # tell distutils packages are under src
    namespace_packages=[],
    include_package_data=True,
    zip_safe=False,
    install_requires=requires,
    url='https://github.com/cbrand/vpnchooser',
    download_url='https://github.com/cbrand/vpnchooser/tarball/%s' % VERSION,
    entry_points={
        'console_scripts': [
            'vpnchooser=vpnchooser.manage:main'
        ]
    },
)



# -*- encoding: utf-8 -*-

from flask.ext.restful import (
    Resource,
    fields,
    marshal_with,
    abort,
    url_for
)
from flask.ext.restful.reqparse import RequestParser

from vpnchooser.syncer import sync
from vpnchooser.helpers import (
    require_login,
    id_from_url
)
from vpnchooser.helpers.fields import AbsoluteUrl, NullableAbsoluteUrl
from vpnchooser.db import session, Device, Vpn


parser = RequestParser()
parser.add_argument(
    'ip', type=str,
    required=True,
)
parser.add_argument(
    'name', type=str,
    required=True,
)
parser.add_argument(
    'type', type=str,
    required=False,
)
parser.add_argument(
    'vpn', type=str,
    required=False
)

resource_fields = {
    'id': fields.Integer,
    'ip': fields.String,
    'name': fields.String,
    'type': fields.String,
    'vpn': NullableAbsoluteUrl('vpn', data_func=lambda obj: {
        'vpn_id': obj.vpn_id
    }),
    'self': AbsoluteUrl('device', data_func=lambda obj: {
        'device_id': obj.id
    }),
}


class AbstractDeviceResource(Resource):
    """
    Abstract of the resource.
    """

    @staticmethod
    def _update_vpn(args, device: Device) -> Device:
        orig_vpn = device.vpn
        if args.vpn:
            try:
                vpn_id = id_from_url(args.vpn, 'vpn_id')
                device.vpn = session.query(Vpn).filter(
                    Vpn.id == vpn_id
                ).first()
            except ValueError:
                device.vpn = None
        else:
            device.vpn = None

        if orig_vpn != device.vpn:
            sync.delay()
        return device

    def update(self, device: Device) -> Device:
        args = parser.parse_args()
        device.ip = args.ip
        device.name = args.name
        device.type = args.type
        self._update_vpn(args, device)
        return device


class DeviceResource(AbstractDeviceResource):
    """
    The resource to access a device resource.
    """

    @staticmethod
    def _get_by_id(device_id: int) -> Device:
        return session.query(Device).filter(
            Device.id == device_id
        ).first()

    def _get_or_abort(self, device_id: int):
        device = self._get_by_id(device_id)
        if device is None:
            abort(404)
        else:
            pass
        return device

    @require_login
    @marshal_with(resource_fields)
    def get(self, device_id: int) -> Device:
        """
        Gets the Device Resource.
        """
        return self._get_or_abort(device_id)

    @require_login
    @marshal_with(resource_fields)
    def put(self, device_id: int) -> Device:
        """
        Updates the Device Resource with the
        name.
        """
        device = self._get_or_abort(device_id)
        self.update(device)
        session.commit()
        session.add(device)
        return device

    @require_login
    def delete(self, device_id: int):
        """
        Deletes the resource with the given name.
        """
        device = self._get_or_abort(device_id)
        session.delete(device)
        session.commit()
        return '', 204


class DeviceListResource(AbstractDeviceResource):
    """
    List resource for the device.
    """

    @require_login
    @marshal_with(resource_fields)
    def get(self):
        devices = list(session.query(Device))
        return devices

    @require_login
    @marshal_with(resource_fields)
    def post(self) -> Device:
        device = Device()
        session.add(device)
        self.update(device)
        session.flush()
        session.commit()
        return device, 201, {
            'Location': url_for('device', device_id=device.id)
        }



# -*- encoding: utf-8 -*-

from flask import Flask, redirect, url_for
from flask.ext.restful import Api

from .celery import make_celery


app = Flask('vpnchooser')
api = Api(app)

celery = make_celery(app)


@app.route('/')
@app.route('/static')
@app.route('/static/')
def index_route():
    return redirect(url_for('static', filename='index.html'))



# -*- encoding: utf-8 -*-

from flask.ext.restful import (
    Resource,
    fields,
    marshal_with,
    abort,
    url_for
)
from flask.ext.restful.reqparse import RequestParser

from vpnchooser.helpers import (
    require_admin, require_login
)
from vpnchooser.helpers.fields import AbsoluteUrl
from vpnchooser.db import session, Vpn


parser = RequestParser()
parser.add_argument(
    'name', type=str,
    required=True,
    help='The name of the vpn.'
)
parser.add_argument(
    'description', type=str,
    required=True,
)
parser.add_argument(
    'table', type=str,
    required=True,
)

resource_fields = {
    'id': fields.Integer,
    'name': fields.String,
    'description': fields.String,
    'table': fields.String,
    'self': AbsoluteUrl('vpn', data_func=lambda obj: {
        'vpn_id': obj.id
    }),
}


class AbstractVpnResource(Resource):
    """
    Abstract of the resource.
    """

    @staticmethod
    def update(vpn: Vpn) -> Vpn:
        args = parser.parse_args()
        vpn.name = args.name
        vpn.description = args.description
        vpn.table = args.table
        return vpn


class VpnResource(AbstractVpnResource):
    """
    The resource to access a vpn resource.
    """

    @staticmethod
    def _get_by_name(vpn_id: int) -> Vpn:
        return session.query(Vpn).filter(
            Vpn.id == vpn_id
        ).first()

    def _get_or_abort(self, vpn_id: int):
        vpn = self._get_by_name(vpn_id)
        if vpn is None:
            abort(404)
        else:
            pass
        return vpn

    @require_login
    @marshal_with(resource_fields)
    def get(self, vpn_id: int) -> Vpn:
        """
        Gets the VPN Resource.
        """
        return self._get_or_abort(vpn_id)

    @require_admin
    @marshal_with(resource_fields)
    def put(self, vpn_id: int) -> Vpn:
        """
        Updates the Vpn Resource with the
        name.
        """
        vpn = self._get_or_abort(vpn_id)
        self.update(vpn)
        session.commit()
        return vpn

    @require_admin
    def delete(self, vpn_id: int):
        """
        Deletes the resource with the given name.
        """
        vpn = self._get_or_abort(vpn_id)
        session.delete(vpn)
        session.commit()
        return '', 204


class VpnListResource(AbstractVpnResource):
    """
    List resource for the vpn.
    """

    @require_login
    @marshal_with(resource_fields)
    def get(self):
        return list(session.query(Vpn))

    @require_admin
    @marshal_with(resource_fields)
    def post(self) -> Vpn:
        """
        Creates the vpn with the given data.
        """
        vpn = Vpn()
        session.add(vpn)
        self.update(vpn)
        session.flush()
        session.commit()
        return vpn, 201, {
            'Location': url_for('vpn', vpn_id=vpn.id)
        }
